@@ -1,23 +1,20 @@
 package Text::Autoformat;
-$Text::Autoformat::VERSION = '1.71_01';
-use 5.006;
-use strict;
-use warnings;
-use Carp;
+
+use strict; use vars qw($VERSION @ISA @EXPORT @EXPORT_OK); use Carp;
+use 5.005;
+use version; $VERSION = qv('1.666.0');
 
 require Exporter;
 
 use Text::Reform qw( form tag break_at break_with break_wrap break_TeX );
-use Text::Autoformat::Hang;
-use Text::Autoformat::NullHang;
 
-our @ISA = qw(Exporter);
-our @EXPORT = qw( autoformat );
-our @EXPORT_OK =
+@ISA = qw(Exporter);
+@EXPORT = qw( autoformat );
+@EXPORT_OK =
     qw( form tag break_at break_with break_wrap break_TeX ignore_headers );
 
 
-my %std_highlight_ignore = map {$_=>1} qw {
+my %ignore = map {$_=>1} qw {
     a an at as and are
     but by 
     ere
@@ -31,11 +28,6 @@ my %std_highlight_ignore = map {$_=>1} qw {
     with while whilst within without
 };
 
-my $STD_HIGHLIGHT_IGNORES = sub {
-    my ($word) = @_;
-    return $std_highlight_ignore{lc $word} ? recase($word,'lower') : recase($word,'title');
-};
-
 my @entities = qw {
     &Aacute;   &aacute;      &Acirc;    &acirc;        &AElig;    &aelig;
     &Agrave;   &agrave;      &Alpha;    &alpha;        &Atilde;   &atilde;
@@ -97,9 +89,9 @@ sub ignore_headers { $_[0] && /$ignore_headers/ }
 
 # BITS OF A TEXT LINE
 
-my $quotechar = qq{[!#%=|:;]};
+my $quotechar = qq{[!#%=|:]};
 my $quotechunk = qq{(?:$quotechar(?![a-z])|(?:[a-z]\\w*)?>+)};
-my $QUOTER = qq{(?:(?i)(?:$quotechunk(?:[ \\t]*$quotechunk)*))};
+my $quoter = qq{(?:(?i)(?:$quotechunk(?:[ \\t]*$quotechunk)*))};
 
 my $separator = q/(?:[-_]{2,}|[=#*]{3,}|[+~]{4,})/;
 
@@ -124,11 +116,12 @@ sub autoformat  # ($text, %args)
         $toSTDOUT = !defined wantarray();
     }
 
+    return unless length $text;
     return $text unless $text =~ /\S/;
 
     $args{right}   = $default_margin unless exists $args{right};
     $args{justify} = "" unless exists $args{justify};
-    $args{widow}   = 0 if ($args{justify}||"") =~ /full/;
+    $args{widow}   = 0 if $args{justify}||"" =~ /full/;
     $args{widow}   = $default_widow unless exists $args{widow};
     $args{case}    = '' unless exists $args{case};
     $args{lists}   = 1 unless exists $args{lists};
@@ -146,26 +139,29 @@ sub autoformat  # ($text, %args)
     # SPECIAL IGNORANCE...
     if ($args{ignore}) {
         $args{all} = 1;
-        $args{ignore} = _build_ignore( $args{ignore} );
+        my $ig_type = ref $args{ignore};
+        if ($ig_type eq 'Regexp') {
+            my $regex = $args{ignore};
+            $args{ignore} = sub { /$regex/ };
+        }
+        elsif ($args{ignore} =~ /^indent/i) {
+            $args{ignore} = sub { ignore_headers(@_) || /$ignore_indent/ };
+        }
+        croak "Expected suboutine reference as value for -ignore option"
+            if ref $args{ignore} ne 'CODE';
+    }
+    elsif ($args{mail}) {
+        $args{ignore} = \&ignore_headers;
     }
     else {
         $args{ignore} = sub{0};
     }
-
-    if ( $args{mail} ) {
-        my $ignore = $args{ignore};
-        $args{ignore} = sub { $ignore->(@_) || ignore_headers(@_) };
-    }
     
     # DETABIFY
     my @rawlines = split /\n/, $text;
     use Text::Tabs; $tabstop = $args{tabspace};
     @rawlines = expand(@rawlines);
 
-    # HANDLE QUOTING CHANGE
-    my $quoter = exists $args{quoter} ? $args{quoter} : $QUOTER;
-    $quoter =~ s/<QUOTER>/$QUOTER/g;
-
     # PARSE EACH LINE
 
     my $pre = 0;
@@ -179,7 +175,7 @@ sub autoformat  # ($text, %args)
             $lines[-1]{presig} .= $lines[-1]{quoter}     = defn $2;
             $lines[-1]{presig} .= $lines[-1]{quotespace} = defn $3;
 
-            $lines[-1]{hang} = Text::Autoformat::Hang->new($_, $args{lists});
+            $lines[-1]{hang} = Hang->new($_, $args{lists});
 
             s/([ \t]*)(.*?)(\s*)$//
                 or die "Internal Error ($@) on '$_'";
@@ -214,6 +210,7 @@ sub autoformat  # ($text, %args)
 
     CHUNK: foreach my $chunk ( @chunks )
     {
+        $DB::single=1;
         next CHUNK if !$args{autocentre} || @$chunk < 2 || $chunk->[0]{hang};
         my @length;
         my $ave = 0;
@@ -296,22 +293,19 @@ sub autoformat  # ($text, %args)
     if ($args{case}) {
         foreach my $para ( @paras ) {
             next if $para->{ignore};
-            if (ref $args{case} eq 'CODE') {
-                $para->{text} = entitle($para->{text}, $args{case});
-            }
-            elsif ($args{case} =~ /upper/i) {
+            if ($args{case} =~ /upper/i) {
                 $para->{text} = recase($para->{text}, 'upper');
             }
-            elsif ($args{case} =~ /lower/i) {
+            if ($args{case} =~ /lower/i) {
                 $para->{text} = recase($para->{text}, 'lower');
             }
-            elsif ($args{case} =~ /title/i) {
-                $para->{text} = entitle($para->{text}, 0);
+            if ($args{case} =~ /title/i) {
+                entitle($para->{text},0);
             }
-            elsif ($args{case} =~ /highlight/i) {
-                $para->{text} = entitle($para->{text}, $STD_HIGHLIGHT_IGNORES);
+            if ($args{case} =~ /highlight/i) {
+                entitle($para->{text},1);
             }
-            elsif ($args{case} =~ /sentence(\s*)/i) {
+            if ($args{case} =~ /sentence(\s*)/i) {
                 my $trailer = $1;
                 $args{squeeze}=0 if $trailer && $trailer ne " ";
                 ensentence();
@@ -465,9 +459,8 @@ sub autoformat  # ($text, %args)
                     $tryformat, @hang,
                     $para->{text});
             $firsttext ||= $newtext;
-            (my $widow) = $newtext =~ /([^\n]*)$/;
-            $widow =~ s/^\s+//;
-            $widow_okay = $para->{empty} || length($widow) >= $args{widow};
+            $newtext =~ /\s*([^\n]*)$/;
+            $widow_okay = $para->{empty} || length($1) >= $args{widow};
             } until $widow_okay || ++$rightindent > $rightslack;
     
             $text .= $widow_okay ? $newtext : $firsttext;
@@ -482,30 +475,7 @@ sub autoformat  # ($text, %args)
     return $text . $remainder;
 }
 
-sub _build_ignore {
-    my $ignore_arg = shift;
-    my $ig_type = ref $ignore_arg;
-    my $ignore;
-    if ($ig_type eq 'Regexp') {
-        my $regex = $ignore_arg;
-        $ignore = sub { /$regex/ };
-    } elsif ($ig_type eq 'ARRAY') {
-        my @elements = map { _build_ignore($_) } @$ignore_arg;
-        $ignore = sub {
-            for my $sub (@elements) { return 1 if $sub->(@_) }
-            return 0;
-        };
-    }
-    elsif ($ignore_arg =~ /^indent/i) {
-        $ignore = sub { ignore_headers(@_) || /$ignore_indent/ };
-    }
-    else {
-        $ignore = $ignore_arg;
-    }
-    croak "Expected suboutine reference as value for -ignore option"
-        if ref $ignore ne 'CODE';
-    return $ignore;
-}
+use utf8;
 
 my $alpha = qr/[^\W\d_]/;
 my $notalpha = qr/[\W\d_]/;
@@ -520,6 +490,7 @@ sub recase {
 
     my $text = "";
     my @pieces = split /(&[a-z]+;)/i, $origtext;
+    use Data::Dumper 'Dumper';
     push @pieces, "" if @pieces % 2;
     return $text unless @pieces;
     local $_ = shift @pieces;
@@ -541,28 +512,38 @@ sub recase {
 my $alword = qr{(?:\pL|&[a-z]+;)(?:[\pL']|&[a-z]+;)*}i;
 
 sub entitle {
-    my ($text, $retitler_ref) = @_;
+    my $ignore = pop;
+    local *_ = \shift;
 
     # put into lowercase if on stop list, else titlecase
-    $text =~ s{($alword)}
-              { $retitler_ref ? $retitler_ref->($1) : recase($1,'title') }gex;
+    s{($alword)}
+     { $ignore && $ignore{lc $1} ? recase($1,'lower') : recase($1,'title') }gex;
 
-    if ($retitler_ref == $STD_HIGHLIGHT_IGNORES) {
-        # First and final words always capitalized...
-        $text =~ s/^($alword) /recase($1,'title')/ex;
-        $text =~ s/ ($alword)$/recase($1,'title')/ex;
+    s/^($alword) /recase($1,'title')/ex;  # last word always to cap
+    s/ ($alword)$/recase($1,'title')/ex;  # first word always to cap
 
-        # treat parethesized portion as a complete title
-        $text =~ s/\( ($alword) /'('.recase($1,'title')/ex;
-        $text =~ s/($alword) \) /recase($1,'title').')'/ex;
+    # treat parethesized portion as a complete title
+    s/\( ($alword) /'('.recase($1,'title')/ex;
+    s/($alword) \) /recase($1,'title').')'/ex;
 
-        # capitalize first word following colon or semi-colon
-        $text =~ s/ ( [:;] \s+ ) ($alword) /$1 . recase($2,'title')/ex;
-    }
-
-    return $text;
+    # capitalize first word following colon or semi-colon
+    s/ ( [:;] \s+ ) ($alword) /$1 . recase($2,'title')/ex;
 }
 
+sub inv($@) { my ($k, %inv)=shift; for(0..$#_) {$inv{$_[$_]}=$_*$k} %inv } 
+my @unit= ( "" , qw ( I II III IV V VI VII VIII IX ));
+my @ten = ( "" , qw ( X XX XXX XL L LX LXX LXXX XC ));
+my @hund= ( "" , qw ( C CC CCC CD D DC DCC DCCC CM ));
+my @thou= ( "" , qw ( M MM MMM ));
+my %rval= (inv(1,@unit),inv(10,@ten),inv(100,@hund),inv(1000,@thou));
+my $rbpat= join ")(",join("|",reverse @thou), join("|",reverse @hund), join("|",reverse @ten), join("|",reverse @unit);
+my $rpat= join ")(?:",join("|",reverse @thou), join("|",reverse @hund), join("|",reverse @ten), join("|",reverse @unit);
+my $rom = qq/(?:(?=[MDCLXVI])(?:$rpat))/;
+
+my $abbrev = join '|', qw{ etc[.]   pp[.]   ph[.]?d[.] },
+                       "(?!$rom)(?:[A-Z][A-Za-z]+[.])+",
+                       '(?:[A-Z][.])(?:[A-Z][.])+';
+
 my $gen_abbrev = join '|',
     qw{ etc[.]  pp[.]   ph[.]?d[.] },
     '(?:[A-Z][.])(?:[A-Z][.])+',
@@ -598,7 +579,21 @@ sub ensentence {
 # blockquote($text,$para, $format, $tlen, \@hang, \%args);
 sub blockquote {
     my ($dummy, $para, $format, $tlen, $hang, $args) = @_;
-
+=begin other
+    print STDERR "[", join("|", $para->{raw} =~
+/ \A(\s*)       # $1 - leading whitespace (quotation)
+       (["']|``)        # $2 - opening quotemark
+       (.*)         # $3 - quotation
+       (''|\2)      # $4 closing quotemark
+       \s*?\n       # trailing whitespace
+       (\1[ ]+)     # $5 - leading whitespace (attribution)
+       (--|-)       # $6 - attribution introducer
+       ([^\n]*?$)       # $7 - attribution line 1
+       ((\5[^\n]*?$)*)      # $8 - attributions lines 2-N
+       \s*\Z
+     /xsm
+), "]\n";
+=cut
     $para->{text} =~
         / \A(\s*)       # $1 - leading whitespace (quotation)
        (["']|``)        # $2 - opening quotemark
@@ -637,6 +632,220 @@ sub blockquote {
     return 1;
 }
 
+package Hang;
+use strict;
+
+# ROMAN NUMERALS
+
+
+sub fromRoman($)
+{
+    return 0 unless $_[0] =~ /^.*?($rbpat).*$/i;
+    return $rval{uc $1} + $rval{uc $2} + $rval{uc $3} + $rval{uc $4};
+}
+
+sub toRoman($$)
+{
+    my ($num,$example) = @_;
+    return '' unless $num =~ /^([0-3]??)(\d??)(\d??)(\d)$/;
+    my $roman = $thou[$1||0] . $hund[$2||0] . $ten[$3||0] . $unit[$4||0];
+    return $example=~/[A-Z]/ ? uc $roman : lc $roman;
+}
+
+# BITS OF A NUMERIC VALUE
+
+my $num = q/(?:\d{1,3}\b(?!:\d\d\b))/;     # Ignore 8:20 etc.
+my $let = q/[A-Za-z]/;
+my $pbr = q/[[(<]/;
+my $sbr = q/])>/;
+my $ows = q/[ \t]*/;
+my %close = ( '[' => ']', '(' => ')', '<' => '>', "" => '' );
+
+my $hangPS      = qq{(?i:ps:|(?:p\\.?)+s\\b\\.?(?:[ \\t]*:)?)};
+my $hangNB      = qq{(?i:n\\.?b\\.?(?:[ \\t]*:)?)};
+my $hangword    = qq{(?:(?:Note)[ \\t]*:)};
+my $hangbullet  = qq{[*.+-]};
+my $hang        = qq{(?:(?i)(?:$hangNB|$hangword|$hangbullet)(?=[ \t]))};
+
+# IMPLEMENTATION
+
+sub new { 
+    my ($class, $orig, $lists_mode) = @_;
+    return NullHang->new() if !$lists_mode;
+
+    my $origlen = length $orig;
+    my @vals;
+    if ($_[1] =~ s#\A($hangPS)##) {
+        @vals = { type => 'ps', val => $1 }
+    }
+    elsif ($lists_mode =~ /1|bullet/i && $_[1] =~ s#\A($hang)##) {
+        @vals = { type => 'bul', val => $1 }
+    }
+    elsif ($_[1] =~ m#\A\([^\s)]+\s#) {
+        @vals = ();
+    }
+    else {
+        local $^W;
+        my $cut;
+        while (length $_[1]) {
+            last if $_[1] =~ m#\A($ows)($abbrev)#
+                 && (length $1 || !@vals);  # ws-separated or first
+
+            last if $_[1] =~ m{\A $ows $pbr [^$sbr \t]* \s}xms;
+
+            $cut = $origlen - length $_[1];
+            my $pre = $_[1] =~ s#\A($ows$pbr$ows)## ? $1 : "";
+            my $val
+                = ($lists_mode =~ /1|number/i && $_[1] =~ s#\A($num)##)
+                        ? { type=>'num', val=>$1 }
+                : ($lists_mode =~ /1|roman/i && $_[1] =~ s#\A($rom)\b##i)
+                        ? { type=>'rom', val=>$1, nval=>fromRoman($1) }
+                : ($lists_mode =~ /1|alpha/i && $_[1] =~ s#\A($let(?!$let))##i)
+                        ? { type=>'let', val=>$1 }
+                :         { val => "", type => "" };
+            $_[1] = $pre.$_[1] and last unless $val->{val};
+            $val->{post} = $pre && $_[1] =~ s#\A($ows()[.:/]?[$close{$pre}][.:/]?)## && $1
+                             || $_[1] =~ s#\A($ows()[$sbr.:/])## && $1
+                             || "";
+            $val->{pre}  = $pre;
+            $val->{cut}  = $cut;
+            push @vals, $val;
+        }
+        while (@vals && !$vals[-1]{post}) {
+            $_[1] = substr($orig,pop(@vals)->{cut});
+        }
+    }
+
+    # check for orphaned years or unlikely Roman numerals...
+    if (@vals==1 && defined $vals[0]->{post} && $vals[0]->{post} =~ /[\.>)]/) {
+        my $v = $vals[0];
+        if ($v->{type} eq 'num' && $v->{val} >= 1000) {
+            $_[1] = substr($orig,pop(@vals)->{cut});
+        }
+    }
+
+    return NullHang->new if !@vals;
+    bless \@vals, $class;
+} 
+
+sub incr {
+    local $^W;
+    my ($self, $prev, $prevsig) = @_;
+    my $level;
+    # check compatibility
+
+    return unless $prev && !$prev->empty;
+
+    for $level (0..(@$self<@$prev ? $#$self : $#$prev)) {
+        if ($self->[$level]{type} ne $prev->[$level]{type}) {
+            return if @$self<=@$prev;   # no incr if going up
+            $prev = $prevsig;
+            last;
+        }
+    }
+    return unless $prev && !$prev->empty;
+    if ($self->[0]{type} eq 'ps') {
+        my $count = 1 + $prev->[0]{val} =~ s/(p[.]?)/$1/gi;
+        $prev->[0]{val} =~ /^(p[.]?).*(s[.]?[:]?)/;
+        $self->[0]{val} = $1  x $count . $2;
+    }
+    elsif ($self->[0]{type} eq 'bul') {
+        # do nothing
+    }
+    elsif (@$self>@$prev) { # going down level(s)
+        for $level (0..$#$prev) {
+                @{$self->[$level]}{'val','nval'} = @{$prev->[$level]}{'val','nval'};
+        }
+        for $level (@$prev..$#$self) {
+                _reset($self->[$level]);
+        }
+    }
+    else    # same level or going up
+    {
+        for $level (0..$#$self) {
+            @{$self->[$level]}{'val','nval'} = @{$prev->[$level]}{'val','nval'};
+        }
+        _incr($self->[-1])
+    }
+}
+
+sub _incr {
+    local $^W;
+    if ($_[0]{type} eq 'rom') {
+        $_[0]{val} = toRoman(++$_[0]{nval},$_[0]{val});
+    }
+    else {
+        $_[0]{val}++ unless $_[0]{type} eq 'let' && $_[0]{val}=~/Z/i;
+    }
+}
+
+sub _reset {
+    local $^W;
+    if ($_[0]{type} eq 'rom') {
+        $_[0]{val} = toRoman($_[0]{nval}=1,$_[0]{val});
+    }
+    elsif ($_[0]{type} eq 'let') {
+        $_[0]{val} = $_[0]{val} =~ /[A-Z]/ ? 'A' : 'a';
+    }
+    else {
+        $_[0]{val} = 1;
+    }
+}
+
+sub stringify {
+    my ($self) = @_;
+    my ($str, $level) = ("");
+    for $level (@$self) {
+        local $^W;
+        $str .= join "", @{$level}{'pre','val','post'};
+    }
+    return $str;
+} 
+
+sub val {
+    my ($self, $i) = @_;
+    return $self->[$i]{val};
+}
+
+sub fields { return scalar @{$_[0]} }
+
+sub field {
+    my ($self, $i, $newval) = @_;
+    $self->[$i]{type} = $newval if @_>2;
+    return $self->[$i]{type};
+}
+
+sub signature {
+    local $^W;
+    my ($self) = @_;
+    my ($str, $level) = ("");
+    for $level (@$self) {
+        $level->{type} ||= "";
+        $str .= join "", $level->{pre},
+                         ($level->{type} =~ /rom|let/ ? "romlet" : $level->{type}),
+                         $level->{post};
+    }
+    return $str;
+} 
+
+sub length {
+    length $_[0]->stringify
+}
+
+sub empty { 0 }
+
+package NullHang;
+use strict;
+
+sub new       { bless {}, $_[0] }
+sub stringify { "" }
+sub length    { 0 }
+sub incr      {}
+sub empty     { 1 }
+sub signature     { "" }
+sub fields { return 0 }
+sub field { return "" }
+sub val { return "" }
 1;
 
 __END__
@@ -647,7 +856,8 @@ Text::Autoformat - Automatic text wrapping and reformatting
 
 =head1 VERSION
 
-This document describes version 1.70 of Text::Autoformat
+This document describes version 1.13 of Text::Autoformat,
+released May  4, 2005.
 
 =head1 SYNOPSIS
 
@@ -694,7 +904,6 @@ This document describes version 1.70 of Text::Autoformat
     $formatted = autoformat $rawtext, { case => 'sentence' };
     $formatted = autoformat $rawtext, { case => 'title' };
     $formatted = autoformat $rawtext, { case => 'highlight' };
-    $formatted = autoformat $rawtext, { case => \&my_case_func };
 
  # Selective reformatting
 
@@ -851,17 +1060,12 @@ If the value of the C<ignore> option is the string C<'indented'>,
 C<autoformat> will ignore any paragraph in which I<every> line begins with a
 whitespace.
 
-You may also specify multiple C<ignore> options by including them in 
-an array-ref:
-
-        $tidied_mesg = autoformat($messy, {ignore=>[qr/1/,'indented']});
-
 One other special case of ignorance is ignoring mail headers and signature.
 This option is specified using the C<mail> argument:
 
         $tidied_mesg = autoformat($messy_mesg, {mail=>1});
 
-Note that the C<ignore> or C<mail> options automatically imply C<all>.
+Note that the C<mail> option automatically implies C<all>.
 
 
 =head2 Bulleting and (re-)numbering
@@ -997,27 +1201,6 @@ C<autoformat> recognizes the various quoting conventions used in this
 example and treats it as three paragraphs to be independently
 reformatted.
 
-You may also override the default set of recognized quoters by specifying
-a C<'quoter'> argument when calling C<autoformat()>. For example, to
-format lines such as:
-
-        // This is a comment
-        // in the standard C(++)
-        // comment-to-EOL
-        // format
-
-specify:
-
-    autoformat($text, { quoter =E<gt> qr{//} })
-
-Instead of completely replacing the existing set of quoters, you can
-I<extend> them by specifying a pattern that includes the metasequence
-C<< <QUOTER> >>, which is then replaced by the module's standard pattern
-for quoters. So, for example, to add C<//> to the set of existing quoters:
-
-    autoformat($text, { quoter =E<gt> qr{//|<QUOTER>} })
-
-
 Block quotations present a different challenge. A typical formatter
 would render the following quotation:
 
@@ -1142,7 +1325,7 @@ C<"autocentre"> argument false.
 
 The C<autoformat> subroutine can also optionally perform case
 conversions on the text it processes. The C<{case =E<gt> I<type>}>
-argument allows the user to specify six different conversions:
+argument allows the user to specify five different conversions:
 
 =over 4
 
@@ -1194,35 +1377,6 @@ capitalized:
 
         'What I Did on my Summer Vacation in Monterey'
 
-=item C<sub{...}>
-
-If the argument for C<'case'> is a subroutine reference, that subroutine
-is applied to each word and the result replaces the word in the text.
-
-For example, to convert a string to hostage-case:
-
-    my $ransom_note = sub {
-        return join "",                    # ^  Reconcatenate
-               map {/[aeiou]/i ? lc : uc}  # |  uPPeR aND LoWeR each
-               split //,                   # |  Break into chars
-               shift;                      # |  Take argument
-    };
-
-    $text = autoformat($text, {case => $ransom_note });
-    # "FoR eXaMPLe, To CoNVeRT a STRiNG To HoSTaGe-CaSe:"
-
-Or to highlight particular words:
-
-    my @SPECIAL = qw( perl camel wall );
-    sub highlight_specials {
-        my ($word) = @_;
-        return $word ~~ @SPECIAL ? uc($word) : $word;
-    }
-
-    $text = autoformat($text, {case => \&highlight_specials});
-    # "It is easier for a CAMEL to pass through a WALL of PERL..."
-
-
 =back
 
 =head2 Selective reformatting
@@ -1260,10 +1414,6 @@ can change that default using the 'tabspace' option:
 
 The Text::Reform module
 
-=head1 REPOSITORY
-
-L<https://github.com/neilbowers/Text-Autoformat>
-
 =head1 AUTHOR
 
 Damian Conway (damian@conway.org)
